package worldcontrolteam.worldcontrol.tileentity;

import javax.annotation.Nonnull;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.NetworkManager;
import net.minecraft.network.play.server.SPacketUpdateTileEntity;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ITickable;
import net.minecraft.util.NonNullList;
import net.minecraft.util.math.BlockPos;
import net.minecraftforge.items.IItemHandler;
import net.minecraftforge.items.IItemHandlerModifiable;
import net.minecraftforge.items.ItemHandlerHelper;
import worldcontrolteam.worldcontrol.api.card.ICard;
import worldcontrolteam.worldcontrol.blocks.BlockInfoPanel;
import worldcontrolteam.worldcontrol.init.WCContent;
import worldcontrolteam.worldcontrol.inventory.ISlotItemFilter;
import worldcontrolteam.worldcontrol.items.ItemUpgrade;
import worldcontrolteam.worldcontrol.screen.IScreenContainer;
import worldcontrolteam.worldcontrol.api.screen.IScreenElement;
import worldcontrolteam.worldcontrol.utils.RedstoneHelper;
import worldcontrolteam.worldcontrol.utils.WCUtility;

import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by dmf444 on 8/15/2017. Code originally written for World-Control.
 */
public class TileEntityInfoPanel extends TileEntity implements IItemHandler, ITickable, ISlotItemFilter, IItemHandlerModifiable, RedstoneHelper.IRedstoneConsumer  {
    public int color;
    public boolean power;

    public BlockPos origin;
    public BlockPos end;

    public IScreenElement ise;

    public EnumFacing facing;
    private NonNullList<ItemStack> itemStack = NonNullList.func_191197_a(3, ItemStack.field_190927_a);

    private Map<BlockPos, Boolean> validCache = new HashMap<>();

    public TileEntityInfoPanel() {
        this.color = 10;
        this.power = true;

        // debug debug debug todo: fixme: aaaaaaa

        this.setStackInSlot(0, new ItemStack(WCContent.TIME_CARD, 1));
        closeInventory(null);
    }

    public void init() {
        this.facing = (EnumFacing) field_145850_b.func_180495_p(func_174877_v()).func_177228_b().get(BlockInfoPanel.FACING);
        if (origin == null) origin = func_174877_v();
        if (end == null) end = func_174877_v();

        searchForNeighbours();
        updateAllProviders(false);
    }

    public void reInit() {
        updateAllProviders(true);
        searchForNeighbours();
        updateAllProviders(false);
    }

    private boolean isAreaValid(BlockPos o, BlockPos e) {
        for (int x = o.func_177958_n(); x <= e.func_177958_n(); x++) {
            for (int y = o.func_177956_o(); y <= e.func_177956_o(); y++) {
                for (int z = o.func_177952_p(); z <= e.func_177952_p(); z++) {
                    final BlockPos pos1 = new BlockPos(x, y, z);
                    if (validCache.containsKey(pos1)) {
                        if (!validCache.get(pos1)) return false;
                        continue;
                    }
                    if (!(field_145850_b.func_180495_p(pos1).func_177230_c() instanceof IScreenContainer)) {
                        validCache.put(pos1, false);
                        return false;
                    }
                    IScreenContainer c = (IScreenContainer) field_145850_b.func_180495_p(pos1).func_177230_c();
                    if (!(c.getFacing(field_145850_b, pos1) == facing)) {
                        validCache.put(pos1, false);
                        return false;
                    }
                    if (!c.isValid(field_145850_b, pos1)) {
                        validCache.put(pos1, false);
                        return false;
                    }
                    if (c.getOrigin(field_145850_b, pos1) != null && !WCUtility.compareBPos(c.getOrigin(field_145850_b, pos1), func_174877_v())) {
                        validCache.put(pos1, false);
                        return false;
                    }
                    if (c.isAdvanced()) {
                        validCache.put(pos1, false);
                        return false;
                    }
                    validCache.put(pos1, true);
                }
            }
        }
        return true;
    }
    
    public void updateAllProviders(boolean nullify) {
        for (int x = origin.func_177958_n(); x <= end.func_177958_n(); x++) {
            for (int y = origin.func_177956_o(); y <= end.func_177956_o(); y++) {
                for (int z = origin.func_177952_p(); z <= end.func_177952_p(); z++) {
                    final BlockPos pos = new BlockPos(x, y, z);
                    if (field_145850_b.func_180495_p(pos).func_177230_c() instanceof IScreenContainer) {
                        if (nullify) {
                            ((IScreenContainer) (field_145850_b.func_180495_p(pos).func_177230_c())).setOrigin(field_145850_b, pos, null);
                        }
                        else {
                            ((IScreenContainer) (field_145850_b.func_180495_p(pos).func_177230_c())).setOrigin(field_145850_b, pos, func_174877_v());
                        }
                    }
                }
            }
        }
    }


    private boolean inSamePlane(BlockPos c) {
        switch (facing) {
            case UP:
            case DOWN:
                return c.func_177956_o() == origin.func_177956_o();
            case NORTH:
            case SOUTH:
                return c.func_177952_p() == origin.func_177952_p();
            default:
                return c.func_177958_n() == origin.func_177958_n();
        }
    }

    public void tryToAdd(BlockPos aPos) {
        if (!inSamePlane(aPos)) {
            return;
        }
        updateAllProviders(true);
        searchForNeighbours();
        updateAllProviders(false);
    }

    private void searchForNeighbours() {
        validCache = new HashMap<>();
        EnumFacing left = EnumFacing.WEST; // negRelativeX
        EnumFacing down = EnumFacing.DOWN; // negRealtiveY

        switch (facing) {
            case UP:
            case DOWN:
                left = EnumFacing.WEST;
                down = EnumFacing.NORTH;
                break;
            case EAST:
            case WEST:
                left = EnumFacing.NORTH;
                down = EnumFacing.DOWN;
                break;
            case NORTH:
            case SOUTH:
                left = EnumFacing.WEST;
                down = EnumFacing.DOWN;
                break;
        }
        if (!isAreaValid(origin, end)) {
            origin = func_174877_v();
            end = func_174877_v();
        }
        int n = 0;
        do {
            origin = origin.func_177972_a(left);
            n += 1;
        } while (isAreaValid(origin, end.func_177967_a(left, n)));


        n = 0;

        origin = origin.func_177972_a(left.func_176734_d());

        do {
            origin = origin.func_177972_a(down);
            n += 1;
        } while (isAreaValid(origin, end.func_177967_a(down, n)));


        origin = origin.func_177972_a(down.func_176734_d());
        n = 0;

        do {
            end = end.func_177972_a(left.func_176734_d());
            n += 1;
        } while (isAreaValid(origin.func_177967_a(left.func_176734_d(), n), end));


        end = end.func_177972_a(left);
        n = 0;

        do {
            end = end.func_177972_a(down.func_176734_d());
            n += 1;
        } while (isAreaValid(origin.func_177967_a(down.func_176734_d(), n), end));


        end = end.func_177972_a(down);
        IBlockState bs = field_145850_b.func_180495_p(func_174877_v());
        field_145850_b.func_184138_a(func_174877_v(), bs, bs, 0);
    }

    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound compound) {
        compound = super.func_189515_b(compound);

        if (origin == null) origin = func_174877_v();
        if (end == null) end = func_174877_v();
        if (facing == null) facing = EnumFacing.NORTH;

        compound.func_74768_a("oX", origin.func_177958_n());
        compound.func_74768_a("oY", origin.func_177956_o());
        compound.func_74768_a("oZ", origin.func_177952_p());

        compound.func_74768_a("eX", end.func_177958_n());
        compound.func_74768_a("eY", end.func_177956_o());
        compound.func_74768_a("eZ", end.func_177952_p());

        compound.func_74768_a("facing", facing.func_176745_a());
        compound.func_74757_a("power", power);
        compound.func_74768_a("color", color);

        return compound;
    }

    @Override
    public void func_145839_a(NBTTagCompound compound) {
        super.func_145839_a(compound);

        origin = new BlockPos(
                compound.func_74762_e("oX"),
                compound.func_74762_e("oY"),
                compound.func_74762_e("oZ"));

        end = new BlockPos(
                compound.func_74762_e("eX"),
                compound.func_74762_e("eY"),
                compound.func_74762_e("eZ"));

        facing = EnumFacing.func_82600_a(compound.func_74762_e("facing"));
        power = compound.func_74767_n("power");
        color = compound.func_74762_e("color");

        //updateAllProviders(false);
    }

    @Nullable
    @Override
    public SPacketUpdateTileEntity func_189518_D_() {
        NBTTagCompound compound = new NBTTagCompound();

        compound.func_74768_a("oX", origin.func_177958_n());
        compound.func_74768_a("oY", origin.func_177956_o());
        compound.func_74768_a("oZ", origin.func_177952_p());

        compound.func_74768_a("eX", end.func_177958_n());
        compound.func_74768_a("eY", end.func_177956_o());
        compound.func_74768_a("eZ", end.func_177952_p());

        compound.func_74768_a("facing", facing.func_176745_a());
        compound.func_74757_a("power", power);
        compound.func_74768_a("color", color);

        return new SPacketUpdateTileEntity(func_174877_v(), 3, compound);
    }

    @Override
    public NBTTagCompound func_189517_E_() {
        NBTTagCompound compound = super.func_189517_E_();

        compound.func_74768_a("oX", origin.func_177958_n());
        compound.func_74768_a("oY", origin.func_177956_o());
        compound.func_74768_a("oZ", origin.func_177952_p());

        compound.func_74768_a("eX", end.func_177958_n());
        compound.func_74768_a("eY", end.func_177956_o());
        compound.func_74768_a("eZ", end.func_177952_p());

        compound.func_74768_a("facing", facing.func_176745_a());
        compound.func_74757_a("power", power);
        compound.func_74768_a("color", color);

        return compound;
    }

    @Override
    public void onDataPacket(NetworkManager net, SPacketUpdateTileEntity pkt) {
        NBTTagCompound compound = pkt.func_148857_g();

        origin = new BlockPos(
                compound.func_74762_e("oX"),
                compound.func_74762_e("oY"),
                compound.func_74762_e("oZ"));

        end = new BlockPos(
                compound.func_74762_e("eX"),
                compound.func_74762_e("eY"),
                compound.func_74762_e("eZ"));

        facing = EnumFacing.func_82600_a(compound.func_74762_e("facing"));
        power = compound.func_74767_n("power");
        color = compound.func_74762_e("color");
    }


    @Override
    public int getSlots() {
        return this.itemStack.size();
    }

    @Override
    public ItemStack getStackInSlot(int i) {
        return this.itemStack.get(i);
    }

    @Nonnull
    @Override
    public ItemStack insertItem(int slot, @Nonnull ItemStack stack, boolean simulate) {
        //Modified from {@link SidedInvWrapper#extractItem}
        if (stack.func_190926_b())
            return ItemStack.field_190927_a;

        if (slot == -1)
            return stack;

        ItemStack stackInSlot = this.getStackInSlot(slot);

        int m;
        if (!stackInSlot.func_190926_b())
        {
            if (stackInSlot.func_190916_E() >= Math.min(stackInSlot.func_77976_d(), getSlotLimit(slot)))
                return stack;

            if (!ItemHandlerHelper.canItemStacksStack(stack, stackInSlot))
                return stack;


            m = Math.min(stack.func_77976_d(), getSlotLimit(slot)) - stackInSlot.func_190916_E();

            if (stack.func_190916_E() <= m) {
                if (!simulate) {
                    ItemStack copy = stack.func_77946_l();
                    copy.func_190917_f(stackInSlot.func_190916_E());
                    this.itemStack.set(slot, copy);
                }

                return stack;
            } else {
                // copy the stack to not modify the original one
                stack = stack.func_77946_l();
                if (!simulate) {
                    ItemStack copy = stack.func_77979_a(m);
                    copy.func_190917_f(stackInSlot.func_190916_E());
                    this.itemStack.set(slot, copy);
                    return stack;
                } else {
                    stack.func_190918_g(m);
                    return stack;
                }
            }
        } else {

            m = Math.min(stack.func_77976_d(), getSlotLimit(slot));
            if (m < stack.func_190916_E()) {
                // copy the stack to not modify the original one
                stack = stack.func_77946_l();
                if (!simulate) {
                    this.itemStack.set(slot, stack.func_77979_a(m));
                    return stack;
                } else {
                    stack.func_190918_g(m);
                    return stack;
                }
            } else {
                if (!simulate)
                    this.itemStack.set(slot,  stack);
                return ItemStack.field_190927_a;
            }
        }
    }

    @Nonnull
    @Override
    public ItemStack extractItem(int slot, int amount, boolean simulate) {
        //Modified from {@link SidedInvWrapper#extractItem}
        if(amount == 0)
            return ItemStack.field_190927_a;

        ItemStack stack = getStackInSlot(slot);
        if(stack.func_190926_b()){
            return ItemStack.field_190927_a;
        }

        if (simulate)
        {
            if (stack.func_190916_E() < amount)
            {
                return stack.func_77946_l();
            }
            else
            {
                ItemStack copy = stack.func_77946_l();
                copy.func_190920_e(amount);
                return copy;
            }
        }
        else
        {
            int m = Math.min(stack.func_190916_E(), amount);
            ItemStack ret = this.decrStackSize(slot, m);
            this.func_70296_d();
            return ret;
        }
    }

    public ItemStack decrStackSize(int slot, int amount) {
        ItemStack stack = getStackInSlot(slot);
        if (stack != ItemStack.field_190927_a)
            if (stack.func_190916_E() > amount) {
                stack = stack.func_77979_a(amount);
                func_70296_d();
            } else {
                this.itemStack.set(slot, ItemStack.field_190927_a);
            }
        return stack;
    }

    @Override
    public int getSlotLimit(int slot) {
        return 64;
    }


    public void closeInventory(EntityPlayer entityPlayer) {
        if (this.getStackInSlot(0) == ItemStack.field_190927_a) {
            ise = null;
        }
        else {
            ise = ((ICard)getCard().func_77973_b()).getRenderer(getCard());
            ise.setContext(IScreenElement.EnumScreenContext.SCREEN_BASIC, showLabels);

        }
    }

    @Override
    public void func_73660_a() {
        if (getCard() != ItemStack.field_190927_a && ise != null) {
            ICard icard = (ICard) getCard().func_77973_b();
            icard.update(field_145850_b, getCard());
            ise.onCardUpdate(field_145850_b, getCard());
        }
        RedstoneHelper.checkPowered(field_145850_b, this);
    }

    private ItemStack getCard(){
        return this.getStackInSlot(0);
    }

    @Override
    public boolean isItemValid(int slotIndex, ItemStack itemStack) {
        switch (slotIndex) {
            case 0:
                return itemStack.func_77973_b() instanceof ICard;
            case 1:
                return itemStack.func_77973_b() instanceof ItemUpgrade && itemStack.func_77960_j() == ItemUpgrade.DAMAGE_RANGE;
            default:
                return itemStack.func_77973_b() instanceof ItemUpgrade && itemStack.func_77960_j() == ItemUpgrade.DAMAGE_COLOR;
        }

    }

    @Override
    public void setStackInSlot(int slot, @Nonnull ItemStack stack) {
        if (!isItemValid(slot, stack)) return;
        this.itemStack.set(slot, stack);
        if(slot == 0){
            ise = ((ICard)getCard().func_77973_b()).getRenderer(getCard());
            ise.setContext(IScreenElement.EnumScreenContext.SCREEN_BASIC, showLabels);
        }
    }

    /* todo: sync to NBT */
    private boolean showLabels = true;
    public boolean shouldShowLabels(){
        return this.showLabels;
    }

    public void updateShowLabels(boolean update){
        this.showLabels = update;
        if (ise != null) {
            ise.setContext(IScreenElement.EnumScreenContext.SCREEN_BASIC, this.showLabels);
        }
    }

    @Override
    public boolean getPowered(){
      return power;
    }

    @Override
    public void setPowered(boolean value){
      this.power = value;
    }

}
